from __future__ import annotations
from dataclasses import dataclass, field
from typing import Sequence, TYPE_CHECKING, Optional

from ResSimpy.Nexus.DataModels.Network.NexusActivationChange import NexusActivationChange

if TYPE_CHECKING:
    from ResSimpy.Nexus.NexusNetwork import NexusNetwork


@dataclass(kw_only=True)
class NexusActivationChanges:
    """A class representing a list of nexus action objects."""
    __activationChanges: list[NexusActivationChange] = field(default_factory=list)

    def __init__(self, parent_network: NexusNetwork) -> None:
        """Initialises the NexusActivationChanges class.

        Args:
            parent_network (NexusNetwork): The network that the actions are a part of.
        """
        self.__parent_network: NexusNetwork = parent_network
        self.__activationChanges = []

    def get_all(self) -> Sequence[NexusActivationChange]:
        """Ensures the surface network file has been loaded."""
        self.__parent_network.get_load_status()
        return self.__activationChanges

    def _add_to_memory(self, activations_to_add: Optional[list[NexusActivationChange]]) -> None:
        """Adds the list of NexusActivationChange objects to memory."""
        if activations_to_add is None:
            return

        self.__activationChanges.extend(activations_to_add)



"""Class for handling drills in the Nexus Network.

This class is used to store and manipulate the wellheads in a NexusNetwork. It is stored as an instance in the
NexusNetwork class as "drill_sites". In Nexus this is the DRILLSITE table.
"""
from __future__ import annotations

from dataclasses import dataclass, field
from typing import Optional, TYPE_CHECKING
from uuid import UUID

import pandas as pd

from ResSimpy.Enums.UnitsEnum import UnitSystem
from ResSimpy.FileOperations.File import File
from ResSimpy.GenericContainerClasses.DrillSites import DrillSites
from ResSimpy.Nexus.DataModels.Network.NexusDrillSite import NexusDrillSite
from ResSimpy.Nexus.nexus_add_new_object_to_file import AddObjectOperations
from ResSimpy.Nexus.nexus_collect_tables import collect_all_tables_to_objects
from ResSimpy.Nexus.nexus_modify_object_in_file import ModifyObjectOperations
from ResSimpy.Nexus.nexus_remove_object_from_file import RemoveObjectOperations
from ResSimpy.Utils.obj_to_dataframe import obj_to_dataframe

if TYPE_CHECKING:
    from ResSimpy.Nexus.NexusNetwork import NexusNetwork


@dataclass
class NexusDrillSites(DrillSites):
    _drill_sites: list[NexusDrillSite] = field(default_factory=list)

    def __init__(self, parent_network: NexusNetwork) -> None:
        """Initialises the NexusDrillSites class.

        Args:
            parent_network (NexusNetwork): The network that the drill sites are a part of.
        """
        self.__parent_network: NexusNetwork = parent_network
        self._drill_sites: list[NexusDrillSite] = []
        self.__add_object_operations = AddObjectOperations(NexusDrillSite, self.table_header, self.table_footer,
                                                           self.__parent_network.model)
        self.__remove_object_operations = RemoveObjectOperations(self.__parent_network, self.table_header,
                                                                 self.table_footer)
        self.__modify_object_operations = ModifyObjectOperations(self)

    def get_all(self) -> list[NexusDrillSite]:
        """Returns a list of drills loaded from the simulator."""
        self.__parent_network.get_load_status()
        return self._drill_sites

    def get_by_name(self, name: str) -> Optional[NexusDrillSite]:
        """Returns a single drill site with the provided name loaded from the simulator.

        Args:
            name (str): name of the requested drill site
        Returns:
            NexusDrillSite: which has the same name as requested
        """
        to_return = filter(lambda x: False if x.name is None else x.name.upper() == name.upper(),
                           self._drill_sites)
        return next(to_return, None)

    def get_df(self) -> pd.DataFrame:
        """Returns drill site data as a dataframe."""
        return obj_to_dataframe(self._drill_sites)

    def load(self, surface_file: File, start_date: str, default_units: UnitSystem) -> None:
        """Loads drill site data from surface file.

        Args:
            surface_file(file): Nexus file representation of the surface file.
            start_date(str): start date of the run.
            default_units(UnitSystem): default units used if no units are found.
        """
        new_drill_sites, _ = collect_all_tables_to_objects(surface_file, {'DRILLSITE': NexusDrillSite},
                                                           start_date=start_date,
                                                           default_units=default_units,
                                                           date_format=self.__parent_network.model.date_format)
        drill_sites_list = new_drill_sites.get('DRILLSITE')
        self._add_to_memory(drill_sites_list)

    def _add_to_memory(self, additional_list: Optional[list[NexusDrillSite]]) -> None:
        """Extends the drill object by a list of drill sites provided to it.

        Args:
            additional_list (list[NexusDrill]): list of nexus drill sites to add to the drill list.
        """
        if additional_list is None:
            return
        self._drill_sites.extend(additional_list)

    def remove(self, obj_to_remove: dict[str, None | str | float | int] | UUID) -> None:
        """Remove a drill site from the network based on the properties matching a dictionary or id.

        Args:
            obj_to_remove (UUID | dict[str, None | str | float | int]): UUID of the drill site to remove or a dictionary
            with sufficient matching parameters to uniquely identify a drill site

        """
        self.__remove_object_operations.remove_object_from_network_main(
            obj_to_remove, self._network_element_name, self._drill_sites)

    def add(self, obj_to_add: dict[str, None | str | float | int]) -> None:
        """Adds a drill site to a network, taking a dictionary with properties for the new drill site.

        Args:
            obj_to_add (dict[str, None | str | float | int]): dictionary taking all the properties for the new
            drill.
            Requires date and a name.
        """
        new_object = self.__add_object_operations.add_network_obj(obj_to_add, NexusDrillSite, self.__parent_network)
        self._add_to_memory([new_object])

    def modify(self, obj_to_modify: dict[str, None | str | float | int],
               new_properties: dict[str, None | str | float | int]) -> None:
        """Modifies an existing drill site based on a matching dictionary of properties.

        Partial matches allowed if precisely 1 matching node is found.
        Updates the properties with properties in the new_properties dictionary.

        Args:
            obj_to_modify (dict[str, None | str | float | int]): dictionary containing attributes to match in the
            existing drill sites.
            new_properties (dict[str, None | str | float | int]): properties to switch to in the new drill site.
        """
        self.__parent_network.get_load_status()
        self.__modify_object_operations.modify_network_object(obj_to_modify, new_properties,
                                                              self.__parent_network)

    @property
    def table_header(self) -> str:
        """Start of the drill site definition table."""
        return 'DRILLSITE'

    @property
    def table_footer(self) -> str:
        """End of the drill site definition table."""
        return 'END' + self.table_header

    def get_overview(self) -> str:
        """Returns overview of the drill site."""
        raise NotImplementedError('To be implemented')



from __future__ import annotations
from abc import ABC
from dataclasses import dataclass
from typing import Optional

from ResSimpy.Enums.UnitsEnum import UnitSystem
from ResSimpy.DataModelBaseClasses.NetworkObject import NetworkObject
from ResSimpy.Nexus.NexusEnums.DateFormatEnum import DateFormat
from ResSimpy.Units.AttributeMappings.NetworkUnitMapping import NetworkUnits


@dataclass(repr=False)
class Target(NetworkObject, ABC):
    control_quantity: Optional[str] = None
    control_conditions: Optional[str] = None
    control_connections: Optional[str] = None
    control_method: Optional[str] = None
    calculation_method: Optional[str] = None
    calculation_conditions: Optional[str] = None
    calculation_connections: Optional[str] = None
    value: Optional[float] = None
    add_value: Optional[float] = None
    region: Optional[str] = None
    priority: Optional[int] = None
    minimum_rate: Optional[str] = None
    minimum_rate_no_shut: Optional[float] = None
    guide_rate: Optional[str] = None
    max_change_pressure: Optional[float] = None
    rank_dt: Optional[float] = None
    control_type: Optional[str] = None
    calculation_type: Optional[str] = None
    region_number: Optional[int] = None

    def __init__(self, properties_dict: Optional[dict[str, None | int | str | float]] = None,
                 date: Optional[str] = None, date_format: Optional[DateFormat] = None, start_date: Optional[str] = None,
                 unit_system: Optional[UnitSystem] = None, name: Optional[str] = None,
                 control_quantity: Optional[str] = None, control_conditions: Optional[str] = None,
                 control_connections: Optional[str] = None, control_method: Optional[str] = None,
                 calculation_method: Optional[str] = None, calculation_conditions: Optional[str] = None,
                 calculation_connections: Optional[str] = None, value: Optional[float] = None,
                 add_value: Optional[float] = None, region: Optional[str] = None, priority: Optional[int] = None,
                 minimum_rate: Optional[str] = None, minimum_rate_no_shut: Optional[float] = None,
                 guide_rate: Optional[str] = None, max_change_pressure: Optional[float] = None,
                 rank_dt: Optional[float] = None, control_type: Optional[str] = None,
                 calculation_type: Optional[str] = None, region_number: Optional[int] = None) -> None:
        """Initialises the Target class.

        Args:
            properties_dict (dict): dict of the properties to set on the object.
            date (Optional[str]): The date of the object.
            date_format (Optional[DateFormat]): The date format that the object uses.
            start_date (Optional[str]): The start date of the model. Required if the object uses a decimal TIME.
            unit_system (Optional[UnitSystem]): The unit system of the object e.g. ENGLISH, METRIC.
            name (Optional[str]): The name of the object.
            control_quantity(Optional[str]): Specifies control type for connections.
            control_conditions(Optional[str]): Column heading keyword indicating the conditions used for control.
            control_connections(Optional[str]): Specifies connections controlled to meet targets, using lists or single
            connections; constraints ensure targets are satisfied.
            control_method(Optional[str]): Indicates the method used to allocate the target constraint to controlling
            connections.
            calculation_method (Optional[str]): Specifies how the target rate is determined.
            calculation_conditions (Optional[str]): Indicates the conditions used to calculate the target rate.
            calculation_connections (Optional[str]): Indicates which connections contribute to the target calculation.
            value (Optional[float]): Indicates the value for the target rate.
            add_value (Optional[float]): Specifies an amount to be added to the target rate calculated by CALCMETHOD.
            region (Optional[str]): Specified a region name from REGDATA table.
            priority (Optional[int]): Column heading for target priority; lower integers indicate higher priority.
            minimum_rate (Optional[str]): Indicates the minimum rate for each connection. Applies to all CTRL
            unless an individual qmin is specified for a connections in a subsequent TGTCON table.
            minimum_rate_no_shut (Optional[float]): Indicates the minimum rate for each connection. Applies to all CTRL
            unless an individual qmin_noshut is specified for a connections in a subsequent TGTCON table.
            guide_rate (Optional[str]): Specifies the guide rate for each connection.
            max_change_pressure (Optional[float]): Specifies the maximum rate of change of region pressure versus time.
            rank_dt (Optional[float]): Specifies the minimum time change between reranking the connections.
            control_type (Optional[str]): Specifies which connections (or wells) should be included from CTRLCONS
            connections.
            calculation_type (Optional[str]): Specifies which connections (or wells) should be included from CALCONS
            connections.
            region_number (Optional[int]): Region numbers, to which function input options are to be applied.
        """
        self.control_quantity = control_quantity
        self.control_conditions = control_conditions
        self.control_connections = control_connections
        self.control_method = control_method
        self.calculation_method = calculation_method
        self.calculation_conditions = calculation_conditions
        self.calculation_connections = calculation_connections
        self.value = value
        self.add_value = add_value
        self.region = region
        self.priority = priority
        self.minimum_rate = minimum_rate
        self.minimum_rate_no_shut = minimum_rate_no_shut
        self.guide_rate = guide_rate
        self.max_change_pressure = max_change_pressure
        self.rank_dt = rank_dt
        self.control_type = control_type
        self.calculation_type = calculation_type
        self.region_number = region_number

        super().__init__(properties_dict=properties_dict, date=date, date_format=date_format, start_date=start_date,
                         unit_system=unit_system, name=name)

    @property
    def units(self) -> NetworkUnits:
        """Returns the attribute to unit map for the WellConnection."""
        return NetworkUnits(self.unit_system)



from dataclasses import dataclass
from abc import ABC
from typing import Mapping

from ResSimpy.DataModelBaseClasses.DynamicProperty import DynamicProperty


@dataclass(kw_only=True)
class Gaslift(ABC):
    """The abstract base class for a collection of gaslift property inputs.

    Attributes:
        inputs (dict[int, DynamicProperty]): Collection of gaslift property inputs, as a dictionary.
    """

    @property
    def inputs(self) -> Mapping[int, DynamicProperty]:
        """Returns mapping of gaslift property inputs."""
        raise NotImplementedError("Implement this in the derived class")



"""Abstract base class for holding the network data for holding all components of the production networks."""

from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Optional

from ResSimpy.GenericContainerClasses.Constraints import Constraints
from ResSimpy.GenericContainerClasses.NetworkLists import NetworkLists
from ResSimpy.GenericContainerClasses.NodeConnections import NodeConnections
from ResSimpy.GenericContainerClasses.Nodes import Nodes
from ResSimpy.GenericContainerClasses.Targets import Targets
from ResSimpy.GenericContainerClasses.WellConnections import WellConnections
from ResSimpy.GenericContainerClasses.Wellbores import Wellbores
from ResSimpy.GenericContainerClasses.Wellheads import Wellheads


@dataclass(kw_only=True, init=False)
class Network(ABC):
    """Abstract base class for holding the network data, holding all components of the production networks."""
    nodes: Optional[Nodes]
    connections: Optional[NodeConnections]
    constraints: Optional[Constraints]
    targets: Optional[Targets] = None
    well_connections: Optional[WellConnections] = None
    wellheads: Optional[Wellheads] = None
    wellbores: Optional[Wellbores] = None
    welllists: Optional[NetworkLists] = None
    _has_been_loaded: bool = False

    def __init__(self, nodes: Optional[Nodes] = None, connections: Optional[NodeConnections] = None,
                 constraints: Optional[Constraints] = None, targets: Optional[Targets] = None,
                 well_connections: Optional[WellConnections] = None, wellheads: Optional[Wellheads] = None,
                 wellbores: Optional[Wellbores] = None, welllists: Optional[NetworkLists] = None,
                 assume_loaded: bool = False) -> None:
        """Initialising the Network Abstract Base class."""
        self._has_been_loaded = assume_loaded
        self.nodes = nodes
        self.connections = connections
        self.constraints = constraints
        self.well_connections = well_connections
        self.targets = targets
        self.wellbores = wellbores
        self.wellheads = wellheads
        self.welllists = welllists

    @abstractmethod
    def load(self) -> None:
        """Loads Network data."""
        raise NotImplementedError("Implement this in the derived class")

    def get_load_status(self) -> bool:
        """Checks load status and loads the network if it hasn't already been loaded."""
        if not self._has_been_loaded:
            self.load()
        return self._has_been_loaded



from __future__ import annotations
from abc import ABC
from dataclasses import dataclass
from typing import Optional

from ResSimpy.Enums.UnitsEnum import UnitSystem
from ResSimpy.DataModelBaseClasses.NetworkObject import NetworkObject
from ResSimpy.Nexus.NexusEnums.DateFormatEnum import DateFormat
from ResSimpy.Units.AttributeMappings.NetworkUnitMapping import NetworkUnits


@dataclass(repr=False)
class WellConnection(NetworkObject, ABC):
    bhdepth: Optional[float] = None
    datum_depth: Optional[float] = None
    x_pos: Optional[float] = None
    y_pos: Optional[float] = None
    length: Optional[float] = None
    temperature: Optional[float] = None
    diameter: Optional[float] = None
    roughness: Optional[float] = None
    inner_diameter: Optional[float] = None
    productivity_index: Optional[float] = None
    hyd_method: Optional[str] = None
    crossflow: Optional[str] = None
    crossshut: Optional[str] = None
    inj_mobility: Optional[str] = None
    polymer: Optional[str] = None
    stream: Optional[str] = None
    group: Optional[str] = None
    i: Optional[int] = None
    j: Optional[int] = None
    drainage_radius: Optional[float] = None
    pvt_method: Optional[int] = None
    d_factor: Optional[float] = None
    on_time: Optional[float] = None
    well_index_mult: Optional[float] = None

    def __init__(self, properties_dict: Optional[dict[str, None | int | str | float]] = None,
                 date: Optional[str] = None, date_format: Optional[DateFormat] = None, start_date: Optional[str] = None,
                 unit_system: Optional[UnitSystem] = None, name: Optional[str] = None,
                 datum_depth: Optional[float] = None, crossflow: Optional[str] = None, crossshut: Optional[str] = None,
                 inj_mobility: Optional[str] = None, polymer: Optional[str] = None, stream: Optional[str] = None,
                 group: Optional[str] = None, i: Optional[int] = None, j: Optional[int] = None,
                 drainage_radius: Optional[float] = None, pvt_method: Optional[int] = None,
                 d_factor: Optional[float] = None, on_time: Optional[float] = None, bhdepth: Optional[float] = None,
                 x_pos: Optional[float] = None, y_pos: Optional[float] = None, length: Optional[float] = None,
                 temperature: Optional[float] = None, diameter: Optional[float] = None,
                 inner_diameter: Optional[float] = None, roughness: Optional[float] = None,
                 productivity_index: Optional[float] = None, hyd_method: Optional[str] = None,
                 well_index_mult: Optional[float] = None) -> None:
        """Initialises the WellConnection class.

        Args:
            properties_dict (dict): dict of the properties to set on the object.
            date (Optional[str]): The date of the object.
            date_format (Optional[DateFormat]): The date format that the object uses.
            start_date (Optional[str]): The start date of the model. Required if the object uses a decimal TIME.
            unit_system (Optional[UnitSystem]): The unit system of the object e.g. ENGLISH, METRIC.
            name (Optional[str]): The name of the object.
            crossshut (Optional[str]): Cross-shut identifier.
            crossflow (Optional[str]): Crossflow identifier.
            d_factor (Optional[float]): Non-Darcy D-factor (D).
            datum_depth (Optional[float]): Depth relative to a datum.
            drainage_radius (Optional[float]): The drainage radius.
            group (Optional[str]): The group that the Well Connection belongs to.
            i (Optional[int]): The location of the Well Connection in the i direction.
            j (Optional[int]): The location of the Well Connection in the j direction.
            inj_mobility (Optional[str]): Injection mobility identifier.
            on_time (Optional[float]): On-time duration.
            polymer (Optional[str]): Polymer identifier.
            pvt_method (Optional[int]): PVT method.
            stream (Optional[str]): Stream identifier.
            bhdepth (Optional[float]): Bottomhole depth (BHDEPTH).
            x_pos(Optional[float]): X-coordinate position (X).
            y_pos (Optional[float]): Y-coordinate position (Y).
            length (Optional[float]): Length of the well connection (LENGTH).
            temperature (Optional[float]): Temperature (TEMP).
            diameter (Optional[float]):  Diameter of the well connection (DIAM).
            inner_diameter (Optional[float]): Inner diameter of the well connection (INNERDIAM).
            roughness (Optional[float]): Pipe roughness (ROUGHNESS).
            productivity_index (Optional[float]): Productivity index (PI).
            hyd_method (Optional[str]): Hydraulic method (METHOD).
            well_index_mult (Optional[float]): Well index multiplier (WIMULT).
        """

        self.bhdepth = bhdepth
        self.x_pos = x_pos
        self.y_pos = y_pos
        self.length = length
        self.temperature = temperature
        self.diameter = diameter
        self.inner_diameter = inner_diameter
        self.roughness = roughness
        self.productivity_index = productivity_index
        self.hyd_method = hyd_method
        self.well_index_mult = well_index_mult

        super().__init__(properties_dict=properties_dict, date=date, date_format=date_format, start_date=start_date,
                         unit_system=unit_system, name=name)

        self.datum_depth = datum_depth if datum_depth is not None else self.datum_depth
        self.crossflow = crossflow if crossflow is not None else self.crossflow
        self.crossshut = crossshut if crossshut is not None else self.crossshut
        self.inj_mobility = inj_mobility if inj_mobility is not None else self.inj_mobility
        self.polymer = polymer if polymer is not None else self.polymer
        self.stream = stream if stream is not None else self.stream
        self.group = group if group is not None else self.group
        self.i = i if i is not None else self.i
        self.j = j if j is not None else self.j
        self.drainage_radius = drainage_radius if drainage_radius is not None else self.drainage_radius
        self.pvt_method = pvt_method if pvt_method is not None else self.pvt_method
        self.d_factor = d_factor if d_factor is not None else self.d_factor
        self.on_time = on_time if on_time is not None else self.on_time
        self.well_index_mult = well_index_mult if well_index_mult is not None else self.well_index_mult

    @property
    def units(self) -> NetworkUnits:
        """Returns the attribute to unit map for the WellConnection."""
        return NetworkUnits(self.unit_system)



from __future__ import annotations

from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Optional

from ResSimpy.DataModelBaseClasses.NetworkObject import NetworkObject
from ResSimpy.Enums.UnitsEnum import UnitSystem
from ResSimpy.Nexus.NexusEnums.DateFormatEnum import DateFormat
from ResSimpy.Units.AttributeMappings.NetworkUnitMapping import NetworkUnits


@dataclass(repr=False)
class Drill(NetworkObject, ABC):

    @property
    @abstractmethod
    def total_drill_time(self) -> float:
        """The total time to drill the well."""
        raise NotImplementedError("Implement this in the derived class")

    def __init__(self, unit_system: Optional[UnitSystem] = None, name: Optional[str] = None, date: Optional[str] = None,
                 date_format: Optional[DateFormat] = None, start_date: Optional[str] = None,
                 properties_dict: Optional[dict[str, None | int | str | float]] = None) -> None:
        """Initialises the Drill class.

        Args:
            date (Optional[str]): The date of the object.
            date_format (Optional[DateFormat]): The date format that the object uses.
            start_date (Optional[str]): The start date of the model. Required if the object uses a decimal TIME.
            unit_system (Optional[UnitSystem]): The unit system of the object e.g. ENGLISH, METRIC.
            name (Optional[str]): The name of the object.
            properties_dict (dict): dict of the properties to set on the object.
        """

        super().__init__(unit_system=unit_system, name=name, date=date, date_format=date_format, start_date=start_date,
                         properties_dict=properties_dict)

    @property
    def units(self) -> NetworkUnits:
        """Returns the attribute to unit map for the Drill."""
        return NetworkUnits(self.unit_system)



"""Class that is used to represent a nexus node in the network."""

from __future__ import annotations
from dataclasses import dataclass
from typing import Optional

from ResSimpy.Enums.UnitsEnum import UnitSystem
from ResSimpy.Nexus.NexusEnums.DateFormatEnum import DateFormat
from ResSimpy.DataModelBaseClasses.NetworkObject import NetworkObject
from ResSimpy.Units.AttributeMappings.BaseUnitMapping import BaseUnitMapping


@dataclass(kw_only=True, repr=False)
class NexusStation(NetworkObject):
    """Class that is used to represent a nexus station in the network."""

    number: Optional[int] = None
    level: Optional[int] = None
    parent: Optional[str] = None

    def __init__(
        self,
        properties_dict: Optional[dict[str, None | int | str | float]] = None,
        date: Optional[str] = None,
        date_format: Optional[DateFormat] = None,
        start_date: Optional[str] = None,
        unit_system: Optional[UnitSystem] = None,
        name: Optional[str] = None,
        number: Optional[int] = None,
        level: Optional[int] = None,
        parent: Optional[str] = None,
    ) -> None:
        """Initialises the NexusStation class.

        Args:
            properties_dict (dict): dict of the properties to set on the object.
            date (Optional[str]): The date of the object.
            date_format (Optional[DateFormat]): The date format that the object uses.
            start_date (Optional[str]): The start date of the model.
            unit_system (Optional[UnitSystem]): The unit system of the object e.g. ENGLISH, METRIC.
            name (Optional[str]): The name of the object.
            number (Optional[int]): Index associated with station for at a given level.
            level (Optional[int]): Level associated with the station. Parents are assigned the highest level values.
            parent (Optional[str]): The parent of the station, if applicable.
        """
        self.number = number
        self.level = level
        self.parent = parent

        super().__init__(
            properties_dict=properties_dict,
            date=date,
            date_format=date_format,
            start_date=start_date,
            unit_system=unit_system,
            name=name,
        )

    @staticmethod
    def get_keyword_mapping() -> dict[str, tuple[str, type]]:
        """Gets the mapping of nexus keywords to attribute definitions."""
        keywords = {
            "NAME": ("name", str),
            "NUMBER": ("number", int),
            "LEVEL": ("level", int),
            "PARENT": ("parent", str),
        }
        return keywords

    def update(self, input_dictionary: dict[str, None | float | int | str]) -> None:
        """Updates a node based on a dictionary of attributes."""
        for k, v in input_dictionary.items():
            if v is None:
                continue
            if hasattr(self, "_NexusStation__" + k):
                setattr(self, "_NexusStation__" + k, v)
            elif hasattr(super(), "_Station__" + k):
                setattr(self, "_Station__" + k, v)

    @property
    def units(self) -> BaseUnitMapping:
        """Returns the attribute to unit map for the data object."""
        return BaseUnitMapping(None)



from __future__ import annotations
from abc import ABC
from dataclasses import dataclass
from typing import Optional

from ResSimpy.Enums.UnitsEnum import UnitSystem
from ResSimpy.DataModelBaseClasses.NetworkObject import NetworkObject
from ResSimpy.Nexus.NexusEnums.DateFormatEnum import DateFormat
from ResSimpy.Units.AttributeMappings.NetworkUnitMapping import NetworkUnits


@dataclass(repr=False)
class DrillSite(NetworkObject, ABC):
    max_rigs: Optional[int]

    def __init__(self, date: Optional[str] = None, date_format: Optional[DateFormat] = None,
                 start_date: Optional[str] = None, unit_system: Optional[UnitSystem] = None, name: Optional[str] = None,
                 max_rigs: Optional[int] = None,
                 properties_dict: Optional[dict[str, None | int | str | float]] = None) -> None:
        """Initialises the DrillSite class.

        Args:
            properties_dict (dict): dict of the properties to set on the object.
            date (Optional[str]): The date of the object.
            date_format (Optional[DateFormat]): The date format that the object uses.
            start_date (Optional[str]): The start date of the model. Required if the object uses a decimal TIME.
            unit_system (Optional[UnitSystem]): The unit system of the object e.g. ENGLISH, METRIC.
            name (Optional[str]): The name of the object.
            max_rigs (Optional[int]): The maximum number of rigs allowed at the drill site.
        """
        self.max_rigs = max_rigs

        super().__init__(properties_dict=properties_dict, date=date, date_format=date_format, start_date=start_date,
                         unit_system=unit_system, name=name)

    @property
    def units(self) -> NetworkUnits:
        """Returns the attribute to unit map for the DrillSite."""
        return NetworkUnits(self.unit_system)

    @property
    def site_name(self) -> Optional[str]:
        """Returns the site name of the drill site."""
        return self.name
